@@ -28,7 +28,11 @@ import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.nio.file.StandardCopyOption;
+import java.security.DigestInputStream;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
+import java.util.Formatter;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -68,13 +72,13 @@ public class DownloadUtil {
   }
 
   private static final String BASE_URL = "https://dlcdn.apache.org/opennlp/";
-  private static final String MODELS_UD_MODELS_1_0 = "models/ud-models-1.0/";
+  private static final String MODELS_UD_MODELS_1_1 = "models/ud-models-1.1/";
 
   public static final Map<String, Map<ModelType, String>> available_models;
 
   static {
     try {
-      available_models = new DownloadParser(new URL(BASE_URL + MODELS_UD_MODELS_1_0)).getAvailableModels();
+      available_models = new DownloadParser(new URL(BASE_URL + MODELS_UD_MODELS_1_1)).getAvailableModels();
     } catch (MalformedURLException e) {
       throw new RuntimeException(e);
     }
@@ -83,13 +87,11 @@ public class DownloadUtil {
   /**
    * Triggers a download for the specified {@link DownloadUtil.ModelType}.
    *
-   * @param language The ISO language code of the requested model.
+   * @param language  The ISO language code of the requested model.
    * @param modelType The {@link DownloadUtil.ModelType type} of model.
-   * @param type The class of the resulting model.
-   * @param <T> The generic type which is a subclass of {@link BaseModel}.
-   *
+   * @param type      The class of the resulting model.
+   * @param <T>       The generic type which is a subclass of {@link BaseModel}.
    * @return A model instance of type {@link T}.
-   *
    * @throws IOException Thrown if IO errors occurred or the model is invalid.
    */
   public static <T extends BaseModel> T downloadModel(String language, ModelType modelType,
@@ -113,14 +115,12 @@ public class DownloadUtil {
    * if it does not already exist. If a model to be downloaded already
    * exists in that directory, the model will not be re-downloaded.
    *
-   * @param url The model's {@link URL}.
+   * @param url  The model's {@link URL}.
    * @param type The class of the resulting model {@link T}.
-   * @param <T> The generic type which is a subclass of {@link BaseModel}.
-   *
+   * @param <T>  The generic type which is a subclass of {@link BaseModel}.
    * @return A model instance of type {@link T}.
-   *
    * @throws IOException Thrown if the model cannot be downloaded.
-  */
+   */
   public static <T extends BaseModel> T downloadModel(URL url, Class<T> type) throws IOException {
 
     final Path homeDirectory = Paths.get(System.getProperty("user.home") + "/.opennlp/");
@@ -138,6 +138,9 @@ public class DownloadUtil {
         Files.copy(in, localFile, StandardCopyOption.REPLACE_EXISTING);
       }
 
+      validateModel(new URL(url + ".sha512"), localFile);
+
+
       logger.debug("Download complete.");
     }
 
@@ -148,6 +151,57 @@ public class DownloadUtil {
     }
   }
 
+  /**
+   * Validates the downloaded model.
+   *
+   * @param sha512          the url to get the sha512 hash
+   * @param downloadedModel the model file to check
+   * @throws IOException thrown if the checksum could not be computed
+   */
+  private static void validateModel(URL sha512, Path downloadedModel) throws IOException {
+    // Download SHA512 checksum file
+    String expectedChecksum;
+    try (BufferedReader reader = new BufferedReader(new InputStreamReader(sha512.openStream()))) {
+      expectedChecksum = reader.readLine();
+
+      if (expectedChecksum != null) {
+        expectedChecksum = expectedChecksum.split("\\s")[0].trim();
+      }
+    }
+
+    // Validate SHA512 checksum
+    final String actualChecksum = calculateSHA512(downloadedModel);
+    if (!actualChecksum.equalsIgnoreCase(expectedChecksum)) {
+      throw new IOException("SHA512 checksum validation failed. Expected: "
+          + expectedChecksum + ", but got: " + actualChecksum);
+    }
+  }
+
+  private static String calculateSHA512(Path file) throws IOException {
+    try {
+      MessageDigest digest = MessageDigest.getInstance("SHA-512");
+      try (InputStream fis = Files.newInputStream(file);
+           DigestInputStream dis = new DigestInputStream(fis, digest)) {
+        byte[] buffer = new byte[4096];
+        while (dis.read(buffer) != -1) {
+          // Reading the file to update the digest
+        }
+      }
+      return byteArrayToHexString(digest.digest());
+    } catch (NoSuchAlgorithmException e) {
+      throw new IOException("SHA-512 algorithm not found", e);
+    }
+  }
+
+  private static String byteArrayToHexString(byte[] bytes) {
+    try (Formatter formatter = new Formatter()) {
+      for (byte b : bytes) {
+        formatter.format("%02x", b);
+      }
+      return formatter.toString();
+    }
+  }
+
   @Internal
   static class DownloadParser {
 
@@ -160,7 +214,6 @@ public class DownloadUtil {
     }
 
     Map<String, Map<ModelType, String>> getAvailableModels() {
-
       final Matcher matcher = LINK_PATTERN.matcher(fetchPageIndex());
 
       final List<String> links = new ArrayList<>();
@@ -172,27 +225,58 @@ public class DownloadUtil {
     }
 
     private Map<String, Map<ModelType, String>> toMap(List<String> links) {
-
       final Map<String, Map<ModelType, String>> result = new HashMap<>();
-
       for (String link : links) {
-
         if (link.endsWith(".bin")) {
-          if (link.contains("de-ud")) {
+          if (link.contains("de-ud")) { // German
             addModel("de", link, result);
-          } else if (link.contains("en-ud")) {
+          } else if (link.contains("en-ud")) { // English
             addModel("en", link, result);
-          } else if (link.contains("it-ud")) {
+          } else if (link.contains("it-ud")) { // Italian
             addModel("it", link, result);
-          } else if (link.contains("nl-ud")) {
+          } else if (link.contains("nl-ud")) { // Dutch
             addModel("nl", link, result);
-          } else if (link.contains("fr-ud")) {
+          } else if (link.contains("fr-ud")) { // French
             addModel("fr", link, result);
+          } else if (link.contains("bg-ud")) { // Bulgarian
+            addModel("bg", link, result);
+          } else if (link.contains("cs-ud")) { // Czech
+            addModel("cs", link, result);
+          } else if (link.contains("hr-ud")) { // Croatian
+            addModel("hr", link, result);
+          } else if (link.contains("da-ud")) { // Danish
+            addModel("da", link, result);
+          } else if (link.contains("es-ud")) { // Spanish
+            addModel("es", link, result);
+          } else if (link.contains("et-ud")) { // Estonian
+            addModel("et", link, result);
+          } else if (link.contains("fi-ud")) { // Finnish
+            addModel("fi", link, result);
+          } else if (link.contains("lv-ud")) { // Latvian
+            addModel("lv", link, result);
+          } else if (link.contains("no-ud")) { // Norwegian
+            addModel("no", link, result);
+          } else if (link.contains("pl-ud")) { // Polish
+            addModel("pl", link, result);
+          } else if (link.contains("pt-ud")) { // Portuguese
+            addModel("pt", link, result);
+          } else if (link.contains("ro-ud")) { // Romanian
+            addModel("ro", link, result);
+          } else if (link.contains("ru-ud")) { // Russian
+            addModel("ru", link, result);
+          } else if (link.contains("sr-ud")) { // Serbian
+            addModel("sr", link, result);
+          } else if (link.contains("sk-ud")) { // Slovak
+            addModel("sk", link, result);
+          } else if (link.contains("sl-ud")) { // Slovenian
+            addModel("sl", link, result);
+          } else if (link.contains("sv-ud")) { // Swedish
+            addModel("sv", link, result);
+          } else if (link.contains("uk-ud")) { // Ukrainian
+            addModel("uk", link, result);
           }
         }
-
       }
-
       return result;
     }
 
